@@ -25,6 +25,7 @@
 # HUBzero is a registered trademark of HUBzero Foundation, LLC.
 #
 
+
 import ConfigParser
 import hubzero.config.hubzerositeconfig
 import hubzero.data.db
@@ -32,6 +33,7 @@ import re
 import os
 import json
 
+
 def getDefaultSite():
 	return hubzero.config.hubzerositeconfig.getHubzeroConfigOption("DEFAULT", "site")
 	
@@ -123,6 +125,30 @@ def _writeParamField(componentOption, NewParamValueDictionary, hubName):
 	db.query_rowcount(sql, data)
 
 
+def _writePluginParamField(folder, element, NewParamValueDictionary, hubName):
+	"""
+	Take a dictionary, transform it to properly formatted params string for Joomla, and 
+	update the params field of the jos_extensions table for the specified plugin
+	"""
+	# database config
+	dbPW = getWebConfigDBPassword(hubName)
+	dbUserName = getWebConfigDBUsername(hubName)
+	dbName = getWebConfigDBName(hubName)	
+	db =  hubzero.data.db.MySQLConnection("localhost", dbName, dbUserName, dbPW)
+
+	if hubzero.utilities.misc.JOOMLA_25:
+		parmString = json.dumps(NewParamValueDictionary)
+		sql = "UPDATE `jos_extensions` SET `params`=%s WHERE `type`='plugin' AND `folder`=%s AND `element`=%s"
+	else:
+		parmString = ""
+		for parm in NewParamValueDictionary:
+			parmString += parm + "=" + NewParamValueDictionary[parm] + "\n"
+		sql = "UPDATE `jos_plugins` SET `params`=%s WHERE `folder`=%s AND `element`=%s"
+
+	data = (parmString, folder, element)
+	db.query_rowcount(sql, data)
+
+
 def _buildDictionary(componentOption, hubName):
 	"""
 	Put the parameter string of the database into a dictionary
@@ -163,6 +189,48 @@ def _buildDictionary(componentOption, hubName):
 			parmsDict[n] = v
 
 	return parmsDict
+
+
+def _buildPluginDictionary(folder, element, hubName):
+	"""
+	Put the parameter string of the database into a dictionary
+	String follows the format:
+	
+	param1=value1
+	param2=value2
+	param3=value3
+	
+	"""
+	# database config
+	dbPW = getWebConfigDBPassword(hubName)
+	dbUserName = getWebConfigDBUsername(hubName)
+	dbName = getWebConfigDBName(hubName)	
+	db =  hubzero.data.db.MySQLConnection("localhost", dbName, dbUserName, dbPW)
+	
+	# look up parm string and parse it on newline, for each line, parse it further based on *first* '='
+	if hubzero.utilities.misc.JOOMLA_25:
+		sql = "SELECT `params` from jos_extensions WHERE type='plugin' AND folder=%s AND element=%s"
+	else:
+		sql = "SELECT `params` from jos_plugins WHERE `folder`=%s AND `element`=%s"
+
+	data = (folder,element)
+	paramsString = db.query_selectscalar(sql, data)
+
+	if hubzero.utilities.misc.JOOMLA_25 and paramsString.startswith('{'):
+		try:
+			parmsDict = json.loads(paramsString)
+			return parmsDict
+		except:
+			pass
+
+	parmsDict = {}
+	parms = paramsString.split("\n")
+	for parmline in parms:
+		if parmline: # in case of any blank lines in the params field
+			n,v = parmline.split("=", 1)
+			parmsDict[n] = v
+
+	return parmsDict
 	
 
 def getComponentParam(componentOption, parameterName, hubName = ""):
@@ -177,6 +245,18 @@ def getComponentParam(componentOption, parameterName, hubName = ""):
 		return None	
 
 
+def getPluginParam(folder,element, parameterName, hubName = ""):
+	"""
+	Get individual param value for a component and return to caller
+	"""
+	paramDict = _buildPluginDictionary(folder,element, hubName)
+	
+	if paramDict.has_key(parameterName):
+		return paramDict[parameterName]
+	else:
+		return None	
+
+
 def addComponentParam(componentOption, parameterName, parameterValue, allowOverwrite=True, hubName = ""):
 	"""
 	Add value to the param string in the jos_components table for the specified component.
@@ -193,14 +273,40 @@ def addComponentParam(componentOption, parameterName, parameterValue, allowOverw
 
 	_writeParamField(componentOption, paramDict, hubName)	
 
+
+def addPluginParam(folder, element, parameterName, parameterValue, allowOverwrite=True, hubName = ""):
+	"""
+	Add value to the param string in the jos_components table for the specified component.
+	"""
+	paramDict = _buildPluginDictionary(folder,element,hubName)
+
+	if allowOverwrite:
+		paramDict[parameterName] = parameterValue
+	else:
+		if paramDict.has_key(parameterName):
+			raise Exception("ERROR: addPluginParam called for existing parameter with allowOverwrite = false")
+		else:
+			paramDict[parameterName] = parameterValue
+
+	_writePluginParamField(folder,element, paramDict, hubName)	
+
 	
 def delComponentParam(componentOption, parameterName, hubName = ""):
 	"""
-	Delete value from the param string in the in the jos_components table for the specified component
+	Delete value from the param string in the in the jos_extensions table for the specified component
 	"""
 	paramDict = _buildDictionary(componentOption, hubName)
 	del paramDict[parameterName]
 	_writeParamField(componentOption, paramDict, hubName)
+
+
+def delPluginParam(folder, element, parameterName, hubName = ""):
+	"""
+	Delete value from the param string in the in the jos_extensions table for the specified plugin
+	"""
+	paramDict = _buildPluginDictionary(folder, element, hubName)
+	del paramDict[parameterName]
+	_writePluginParamField(folder, element, paramDict, hubName)
 	
 	
 def setPluginEnabled(pluginFolder, pluginElement, enable, hubName = ""):
@@ -221,6 +327,7 @@ def setPluginEnabled(pluginFolder, pluginElement, enable, hubName = ""):
 	data = (enable, pluginFolder, pluginElement)
 	db.query_rowcount(sql, data)	
 
+
 def getPluginEnabled(pluginFolder, pluginElement, hubName = ""):
 	"""
 	Get published field for the given plugin in the jos_plugins table
